from ftw.mobile import _
from ftw.mobile.interfaces import IMobileButton
from plone import api
from plone.app.layout.viewlets.common import ViewletBase
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getAdapters
from zope.component import getMultiAdapter
from zope.i18n import translate
import json
import os


class MobileButtonViewlet(ViewletBase):

    template = ViewPageTemplateFile('templates/buttons_viewlet.pt')

    # Do not render handlebar template with PageTemplate engine,
    # There's actually nothing to process.
    with open(os.path.join(os.path.dirname(__file__),
                           'templates',
                           'list.html'), 'r') as html_file:
        handlebars_list_html = html_file.read()

    with open(os.path.join(os.path.dirname(__file__),
                           'templates',
                           'navigation.html'), 'r') as html_file:
        handlebars_navigation_html = html_file.read()

    def index(self):
        return self.template()

    def buttons(self):
        buttons = list(getAdapters((self.context, self.request),
                                   IMobileButton))

        buttons.sort(key=self.sort_buttons)

        for name, button in buttons:
            if button.available():
                yield {'html': button.render_button(),
                       'name': name}

    def sort_buttons(self, button):
        return button[1].position()

    def nav_root_url(self):
        return api.portal.get_navigation_root(self.context).absolute_url()

    def current_url(self):
        context_state = getMultiAdapter((self.context, self.request),
                                        name=u'plone_context_state')
        return context_state.canonical_object().absolute_url()

    def translation_strings(self):

        return json.dumps(
            {'label_goto_parent': translate(_(u'label_goto_parent',
                                              default=u'Go to parent content of: '),
                                            context=self.request),
             'label_goto_children': translate(_(u'label_goto_children',
                                                default=u'Go to children of: '),
                                              context=self.request)})



from zope.interface import Interface


class IMobileLayer(Interface):
    """Browserlayer marker interface for ftw.mobile"""


class IMobileButton(Interface):

    def __init__(context, request):
        """Adapts context and request"""

    def label():
        """Label of button"""

    def data():
        """json serializeable data to display"""

    def data_url(self):
        """Url to call for json data"""
        return '#'

    def data_template(self):
        """Defines the template used to render the menu"""

    def position():
        """defines the order of the mobile buttons"""

    def available():
        """defines the order of the mobile buttons"""

    def render_button():
        """Link representation"""



import copy
from ftw.mobile.interfaces import IMobileButton
from ftw.mobile.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from plone.app.multilingual.browser.setup import SetupMultilingualSite
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
import json
import transaction


class TestMultilingualButton(FunctionalTestCase):

    supported_languages = ['de', 'fr', 'en']

    def setUp(self):
        super(TestMultilingualButton, self).setUp()
        self.grant('Manager')

        self.language_tool = getToolByName(self.portal, 'portal_languages')
        for lang in self.supported_languages:
            self.language_tool.addSupportedLanguage(lang)

        pam_setup_tool = SetupMultilingualSite()
        pam_setup_tool.setupSite(self.portal)

        self._set_lang('de')
        self.button = getMultiAdapter(
            (self.portal, self.request),
            IMobileButton,
            name='multilanguage-mobile-button'
        )

    def _set_lang(self, lang='de'):
        self.language_tool.manage_setLanguageSettings(
            lang, self.supported_languages,
            setUseCombinedLanguageCodes=False)
        transaction.commit()

    def test_label(self):
        self.assertEquals(u'de', self.button.label())

    def test_data_template(self):
        self.assertEquals('ftw-mobile-list-template',
                          self.button.data_template())

    def test_position(self):
        self.assertEquals(300, self.button.position())

    def test_data(self):
        self.assertEquals(
            [
                {
                    'label': u'Deutsch',
                    'url': u'http://nohost/plone/@@multilingual-selector/notg/de?set_language=de',
                },
                {
                    'label': u'Fran\xe7ais',
                    'url': u'http://nohost/plone/@@multilingual-selector/notg/fr?set_language=fr',
                },
                {
                    'label': u'English',
                    'url': u'http://nohost/plone/@@multilingual-selector/notg/en?set_language=en',
                }
            ],
            self.button.data()
        )

    @browsing
    def test_rendering(self, browser):
        html = self.button.render_button()
        browser.open_html(html)

        link = browser.css('a').first

        self.assertEquals(u'de', link.text)
        self.assertEquals(u'#', link.attrib['href'])
        self.assertEquals(u'', link.attrib['data-mobile_endpoint'])
        self.assertEquals(u'', link.attrib['data-mobile_startup_cachekey'])
        self.assertEquals(u'ftw-mobile-list-template',
                          link.attrib['data-mobile_template'])

        self.assertTrue(
            isinstance(json.loads(link.attrib['data-mobile_data']), list),
            'Expect valid json data in mobile-data')

    def test_button_is_not_available(self):
        # The button must be here because of the 3 configured languages.
        self.assertEqual(
            True,
            self.button.available()
        )

        # Remove languages, keep only "de".
        langs = copy.deepcopy(self.supported_languages)
        langs.remove('de')
        self.language_tool.removeSupportedLanguages(langs)
        self.assertEqual(
            ['de'],
            self.language_tool.getSupportedLanguages()
        )

        # Make sure the button is no longer available.
        button = getMultiAdapter(
            (self.portal, self.request),
            IMobileButton,
            name='multilanguage-mobile-button'
        )
        self.assertEqual(
            False,
            button.available()
        )



from ftw.mobile.buttons import BaseButton
from ftw.mobile.interfaces import IMobileButton
from ftw.mobile.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from zope.component import getGlobalSiteManager
from zope.component import provideAdapter
from zope.interface import Interface
import transaction


class TestUserButton(FunctionalTestCase):

    def setUp(self):
        super(TestUserButton, self).setUp()
        self.buttonklass = None

    @browsing
    def test_viewlet_is_available(self, browser):
        browser.login().visit()

        self.assertTrue(browser.css('#ftw-mobile-menu-buttons'),
                        'Expect the ftw mobile viewlet on the site.')

    @browsing
    def test_default_buttons(self, browser):
        browser.login().visit()
        self.assertEquals(['Mobile navigation', 'User menu'],
                          browser.css('#ftw-mobile-menu-buttons a').text)

    @browsing
    def test_buttons_at_the_end(self, browser):

        self.register_button('somebutton', position=100000)

        try:
            browser.login().visit()
            self.assertEquals(['Mobile navigation', 'User menu', 'somebutton'],
                              browser.css('#ftw-mobile-menu-buttons a').text)
        finally:
            self.unregister_button('somebutton')

    @browsing
    def test_buttons_at_the_beginning(self, browser):

        self.register_button('somebutton', position=1)

        try:
            browser.login().visit()
            self.assertEquals(['somebutton', 'Mobile navigation', 'User menu'],
                              browser.css('#ftw-mobile-menu-buttons a').text)
        finally:
            self.unregister_button('somebutton')

    @browsing
    def test_buttons_in_the_middle(self, browser):

        self.register_button('somebutton', position=500)

        try:
            browser.login().visit()
            self.assertEquals(['Mobile navigation', 'somebutton', 'User menu'],
                              browser.css('#ftw-mobile-menu-buttons a').text)
        finally:
            self.unregister_button('somebutton')

    @browsing
    def test_button_with_empty_json_result_is_filtered(self, browser):

        self.register_button('somebutton', position=500, data=[])

        try:
            browser.login().visit()
            self.assertEquals(['Mobile navigation', 'User menu'],
                              browser.css('#ftw-mobile-menu-buttons a').text)
        finally:
            self.unregister_button('somebutton')

    def register_button(self, name, position, data=None):

        if data is None:
            data = [{'label': 'dummy', 'url': 'http://url.com'}]

        class SomeButton(BaseButton):
            def label(self):
                return name

            def position(self):
                return position

            def data(self):
                return data

        self.buttonklass = SomeButton

        provideAdapter(SomeButton,
                       adapts=(Interface, Interface),
                       name=name)
        transaction.commit()

    def unregister_button(self, name):

        sm = getGlobalSiteManager()
        sm.unregisterAdapter(self.buttonklass,
                             required=(Interface, Interface),
                             provided=IMobileButton,
                             name=name)
        transaction.commit()



from ftw.mobile.buttons import BaseButton
from ftw.mobile.interfaces import IMobileButton
from plone import api
from plone.app.multilingual.browser.selector import LanguageSelectorViewlet
from zope.interface import implementsOnly


class MultilanguageButton(BaseButton, LanguageSelectorViewlet):

    implementsOnly(IMobileButton)

    def __init__(self, context, request):
        super(MultilanguageButton, self).__init__(context, request)
        self.tool = api.portal.get_tool('portal_languages')
        self.buttons = self.get_buttons()

    def available(self):
        """
        Only render the button if there is more than one language configured
        in Plone because it does not make sense to render the button if
        the user has no other language to switch to.
        """
        return len(self.buttons) > 1

    def data(self):
        return self.buttons

    def label(self):
        return api.portal.get_current_language()

    def position(self):
        return 300

    def get_buttons(self):
        translation_languages = self.languages()
        return [
            dict(url=language['url'], label=language['native'])
            for language in translation_languages
        ]



from ftw.mobile.tests import FunctionalTestCase
from ftw.testbrowser import browsing


class TestHandlebarsTemplates(FunctionalTestCase):

    @browsing
    def test_list_template_is_available(self, browser):
        browser.login().visit()

        self.assertTrue(len(browser.css('#ftw-mobile-list-template')),
                        'Expect templates for lists in DOM.')

    @browsing
    def test_navigation_template_is_available(self, browser):
        browser.login().visit()

        self.assertTrue(len(browser.css('#ftw-mobile-navigation-template')),
                        'Expect templates for navigation in DOM.')



from ftw.mobile.interfaces import IMobileButton
from ftw.mobile.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from plone import api
from zope.component import getMultiAdapter
import json
import transaction


class TestUserButton(FunctionalTestCase):

    def setUp(self):
        super(TestUserButton, self).setUp()

        self.user_button = getMultiAdapter((self.portal, self.request),
                                           IMobileButton,
                                           name="user-mobile-button")

    def test_user_button_label(self):
        self.assertEquals('User menu', self.user_button.label())

    def test_user_button_data_template(self):
        self.assertEquals('ftw-mobile-list-template',
                          self.user_button.data_template())

    def test_user_button_position(self):
        self.assertEquals(1000, self.user_button.position())

    def test_user_button_data(self):
        expect = [
            {u'url': u'http://nohost/plone/dashboard',
             u'label': u'Dashboard'},
            {u'url': u'http://nohost/plone/@@personal-preferences',
             u'label': u'Preferences'},
            {u'url': u'http://nohost/plone/logout',
             u'label': u'Log out'}, ]
        self.assertEquals(expect, (self.user_button.data()))

    @browsing
    def test_user_button_rendering(self, browser):
        html = self.user_button.render_button()
        browser.open_html(html)

        link = browser.css('a').first

        self.assertEquals(u'User menu', link.text)
        self.assertEquals(u'#', link.attrib['href'])
        self.assertEquals(u'', link.attrib['data-mobile_endpoint'])
        self.assertEquals(u'', link.attrib['data-mobile_startup_cachekey'])
        self.assertEquals(u'ftw-mobile-list-template',
                          link.attrib['data-mobile_template'])

        self.assertTrue(
            isinstance(json.loads(link.attrib['data-mobile_data']), list),
            'Expect valid json data in mobile-data')

    @browsing
    def test_user_button_labels_are_translated(self, browser):
        lang_tool = api.portal.get_tool('portal_languages')
        lang_tool.setDefaultLanguage('de')
        transaction.commit()

        browser.login().open()
        link = browser.css('#user-mobile-button a').first
        data = link.attrib.get('data-mobile_data')
        self.assertEquals(
            [{u'url': u'http://nohost/plone/dashboard',
              u'label': u'Pers\xf6nliche Seite'},
             {u'url': u'http://nohost/plone/@@personal-preferences',
              u'label': u'Meine Einstellungen'},
             {u'url': u'http://nohost/plone/logout',
              u'label': u'Abmelden'},
            ],
            json.loads(data))



from ftw.mobile import _
from ftw.mobile.interfaces import IMobileButton
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.interface import implements
from zope.interface import Interface
import json


LINK_TEMPLATE = '''
<a href="{url}"
   data-mobile_endpoint="{endpoint}"
   data-mobile_startup_cachekey="{startup_cachekey}"
   data-mobile_template="{mobile_template}"
   data-mobile_settings='{settings}'
   data-mobile_data='{data}'
   data-mobile_label='{label}'>
    {label}
</a>
'''


class BaseButton(object):
    implements(IMobileButton)
    adapts(Interface, Interface)

    def __init__(self, context, request):
        """Adapts context and request"""
        self.context = context
        self.request = request

    def label(self):
        """Label of button"""
        raise NotImplementedError("Implement a label for the button")

    def data(self):
        """json data to display"""
        return []

    def endpoint(self):
        """Viewname of the mobile navigation endpoint.
        """
        return ''

    def data_template(self):
        return 'ftw-mobile-list-template'

    def position(self):
        raise NotImplementedError("Implement a position (int)")

    def available(self):
        return self.data() or self.endpoint()

    def startup_cachekey(self):
        return ''

    def settings(self):
        return {}

    def render_button(self):
        return LINK_TEMPLATE.format(url='#',
                                    startup_cachekey=self.startup_cachekey(),
                                    mobile_template=self.data_template(),
                                    data=json.dumps(self.data()),
                                    settings=json.dumps(self.settings()),
                                    label=self.label(),
                                    endpoint=self.endpoint())


class UserButton(BaseButton):

    def label(self):
        return _(u"User menu")

    def position(self):
        return 1000

    def data(self):
        """json data to display"""
        context_state = getMultiAdapter((self.context, self.request),
                                        name=u'plone_context_state')

        user_actions = context_state.actions('user')

        def link_data(item):
            return {'url': item.get('url'),
                    'label': translate(item.get('title'),
                                       context=self.request)}
        return map(link_data, user_actions)


class NavigationButton(BaseButton):

    def label(self):
        return _(u"Mobile navigation")

    def position(self):
        return 100

    def data_template(self):
        return 'ftw-mobile-navigation-template'

    def endpoint(self):
        return '@@mobilenav'

    def startup_cachekey(self):
        return (self.context.restrictedTraverse('@@mobilenav')
                .get_startup_cachekey())

    def settings(self):
        return {'show_tabs': True,
                'show_two_levels_on_root': True}



from ftw.builder import Builder
from ftw.builder import create
from ftw.mobile.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from operator import itemgetter
from Products.CMFCore.utils import getToolByName
from zExceptions import Unauthorized


class TestMobileNavigation(FunctionalTestCase):

    @browsing
    def test_startup(self, browser):
        self.maxDiff = None
        self.grant('Manager')
        create(Builder('folder').titled('six').within(
            create(Builder('folder').titled('five').within(
                create(Builder('folder').titled('four').within(
                    create(Builder('folder').titled('three').within(
                        create(Builder('folder').titled('two').within(
                            create(Builder('folder').titled('one'))))))))))))

        create(Builder('folder').titled('f').within(
            create(Builder('folder').titled('e').within(
                create(Builder('folder').titled('d').within(
                    create(Builder('folder').titled('c').within(
                        create(Builder('folder').titled('b').within(
                            create(Builder('folder').titled('a'))))))))))))

        browser.open(view='mobilenav/startup')
        expected_startup_paths = [
            u'/plone/a',
            u'/plone/a/b',
            u'/plone/a/b/c',
            u'/plone/a/b/c/d',
            u'/plone/a/b/c/d/e',
            u'/plone/one',
            u'/plone/one/two',
            u'/plone/one/two/three',
            u'/plone/one/two/three/four',
            u'/plone/one/two/three/four/five',
        ]
        self.assertItemsEqual(
            expected_startup_paths,
            map(itemgetter('absolute_path'), browser.json))

        browser.open(self.portal.one.two, view='mobilenav/startup')
        self.assertItemsEqual(
            expected_startup_paths + [
                u'/plone/one/two/three/four/five/six',
            ],
            map(itemgetter('absolute_path'), browser.json))

        # The "children_loaded" property tells the JavaScript tree store
        # whether the children of a node are expected to be delivered within
        # the same JSON response.
        # This decision is made from a query point of view, thus when the container
        # has no children it may still have a "children_loaded" property.
        # The responses are expected to contain all children of a node, or none.
        self.assertEquals(
            {False: [u'/plone/a/b/c/d',
                     u'/plone/a/b/c/d/e',
                     u'/plone/one/two/three/four/five',
                     u'/plone/one/two/three/four/five/six'],
             True: [u'/plone/a',
                    u'/plone/a/b',
                    u'/plone/a/b/c',
                    u'/plone/one',
                    u'/plone/one/two',
                    u'/plone/one/two/three',
                    u'/plone/one/two/three/four']},

            {True: map(itemgetter('absolute_path'),
                       filter(lambda item: item.get('children_loaded'),
                              sorted(browser.json,
                                     key=itemgetter('absolute_path')))),
             False: map(itemgetter('absolute_path'),
                        filter(lambda item: not item.get('children_loaded'),
                               sorted(browser.json,
                                      key=itemgetter('absolute_path'))))})

    @browsing
    def test_startup_headers(self, browser):
        browser.open(view='mobilenav/startup')
        self.assertDictContainsSubset({'content-type': 'application/json',
                                       'x-theme-disabled': 'True'},
                                      browser.headers)
        self.assertNotIn('cache-control', browser.headers,
                         'No cache headers expected when request has'
                         ' no cache key GET parameter.')

        browser.open(view='mobilenav/startup', data={'cachekey': 'abc123'})
        self.assertEquals(
            'public, max-age=31536000',
            browser.headers.get('cache-control'),
            'Expected public cache control since request is anonymous.')

        browser.login().open(view='mobilenav/startup', data={'cachekey': 'abc123'})
        self.assertEquals(
            'private, max-age=31536000',
            browser.headers.get('cache-control'),
            'Expected private cache control since request is authenticated.')

    @browsing
    def test_startup_item_data(self, browser):
        self.grant('Manager')
        create(Builder('folder')
               .titled('The Folder')
               .having(description='A very nice folder'))

        browser.open(view='mobilenav/startup')
        self.assertEquals(
            [{u'absolute_path': u'/plone/the-folder',
              u'children_loaded': True,
              u'description': u'A very nice folder',
              u'externallink': False,
              u'id': u'the-folder',
              u'title': u'The Folder',
              u'url': u'http://nohost/plone/the-folder'}],
            browser.json)

    @browsing
    def test_startup_with_no_View_permission_on_parent(self, browser):
        """Regression test: the navigation should not break when the current
        user has no View permission on any parent.
        """

        self.grant('Manager')
        wftool = getToolByName(self.portal, 'portal_workflow')
        wftool.setChainForPortalTypes(['Folder'], 'simple_publication_workflow')

        one = create(Builder('folder').titled('Folder One'))
        two = create(Builder('folder').titled('Folder Two').within(one))
        john = create(Builder('user').named('John', 'Doe')
                      .with_roles('Reader', on=two))

        browser.login(john).open(two, view='mobilenav/startup')

        self.assertDictEqual(
            {u'/plone/folder-one': u'Folder One',
             u'/plone/folder-one/folder-two': u'Folder Two'},

            dict(map(lambda item: (item['absolute_path'], item['title']),
                     browser.json)))

    @browsing
    def test_startup_does_not_work_on_unauthorized_content(self, browser):
        """For security it is essential that the `startup` endpoint does
        not work on a context where the current user has no `View` permission.
        This is not protected by ZCML since the `children` endpoint needs
        to be public.

        The `startup` endpoint may provide infos of unpublished content (such
        as the title). This is necessary for the navigation to work and follows
        Plone principal but it should not be possible to misuse the endpoint
        for getting those informations on _any_ content.
        """
        self.grant('Manager')
        wftool = getToolByName(self.portal, 'portal_workflow')
        wftool.setChainForPortalTypes(['Folder'], 'simple_publication_workflow')
        folder = create(Builder('folder').titled('Folder'))
        john = create(Builder('user').named('John', 'Doe'))

        browser.login(john)
        with browser.expect_unauthorized():
            browser.open(folder, view='mobilenav/startup')

    @browsing
    def test_children_endpoint_fetches_two_levels(self, browser):
        self.grant('Manager')
        create(Builder('folder').titled('Five').within(
            create(Builder('folder').titled('Four').within(
                create(Builder('folder').titled('Three').within(
                    create(Builder('folder').titled('Two').within(
                        create(Builder('folder').titled('One'))))))))))

        browser.open(self.portal.one, view='mobilenav/children')
        self.assertItemsEqual(
            [
                u'/plone/one',
                u'/plone/one/two',
                u'/plone/one/two/three',
            ],
            map(itemgetter('absolute_path'), browser.json))

        browser.open(self.portal.one.two, view='mobilenav/children')
        self.assertItemsEqual(
            [
                u'/plone/one/two',
                u'/plone/one/two/three',
                u'/plone/one/two/three/four',
            ],
            map(itemgetter('absolute_path'), browser.json))

    @browsing
    def test_children_item_data(self, browser):
        self.grant('Manager')
        create(Builder('folder')
               .titled('The Folder')
               .having(description='A very nice folder'))

        browser.open(view='mobilenav/children')
        self.assertEquals(
            [{u'absolute_path': u'/plone/the-folder',
              u'description': u'A very nice folder',
              u'externallink': False,
              u'id': u'the-folder',
              u'title': u'The Folder',
              u'url': u'http://nohost/plone/the-folder'}],
            browser.json)

    @browsing
    def test_children_fetching_on_unauthorized_works(self, browser):
        """The JavaScript may fetch children now and then, sometimes even
        on objects on which the current user has no View permission.
        In order for the JavaScript to work properly, this should not fail
        with an unauthorized error but return possible children.

        This might be a situation where the unauthorized object was already
        loaded as parent and the JavaScript is trying to fetch its children,
        which may even be published / visible.
        """

        self.grant('Manager')
        wftool = getToolByName(self.portal, 'portal_workflow')
        wftool.setChainForPortalTypes(['Folder'], 'simple_publication_workflow')

        container = create(Builder('folder').titled('Container'))
        visible_child = create(Builder('folder').titled('visible child')
                               .within(container))
        create(Builder('folder').titled('invisible child').within(container))
        john = create(Builder('user').named('John', 'Doe')
                      .with_roles('Reader', on=visible_child))

        browser.login(john).open(container, view='mobilenav/children')
        self.assertItemsEqual(
            [u'/plone/container/visible-child'],
            map(itemgetter('absolute_path'), browser.json))

    @browsing
    def test_children_loaded_flag_on_prepended_items(self, browser):
        self.grant('Manager')
        create(Builder('folder').titled('1b').within(
            create(Builder('folder').titled('1a')
                                    .having(excludeFromNav=True))))
        create(Builder('folder').titled('2a'))

        browser.open(self.portal.get('1a').get('1b'), view='mobilenav/startup')

        # 1a was prepended, since it was excluded from nav.
        response = browser.json
        self.assertIn('children_loaded', response[0])



from ftw.upgrade import UpgradeStep


class InstallFtwGopip(UpgradeStep):
    """Install ftw.gopip (reindexes getObjPositionInParent!)
    """

    def __call__(self):
        self.install_upgrade_profile()
        self.setup_install_profile('profile-ftw.gopip:default')



from Acquisition import aq_base
from functools import partial
from operator import itemgetter
from pkg_resources import get_distribution
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from Products.CMFPlone.browser.navigation import get_view_url
from Products.Five.browser import BrowserView
from zExceptions import Unauthorized
import hashlib
import json
import logging


LOG = logging.getLogger('ftw.mobile')


def is_external_link(brain):
    if brain.portal_type == 'Link':
        url = brain.getRemoteUrl
        return not url.startswith(api.portal.get().absolute_url())
    else:
        return False


class MobileNavigation(BrowserView):
    """Compute navigation based on global settings and some given paramters.
    The return value is always a nested structure, which can be represented
    as json.

    Node representation:
     {'title': '<String>',
      'description': '<String>',
      'id' <String>:
      'url': '<String>',
      'externallink': '<Boolean>'}
    """

    def startup(self):
        """Return all nodes relevant for starting up a mobile navigation
        on the current context.
        """
        if not api.user.has_permission('View', obj=self.context):
            raise Unauthorized()

        response = self.request.response
        response.setHeader('Content-Type', 'application/json')
        response.setHeader('X-Theme-Disabled', 'True')

        filter_exclude_from_nav = self.request.get('ignore_exclude_from_nav') != ''

        if self.request.get('cachekey'):
            # Only cache when there is a cache_key in the request.
            # Representations may be cached by any cache.
            # The cached representation is to be considered fresh for 1 year
            # http://stackoverflow.com/a/3001556/880628
            if api.user.is_anonymous():
                visibility = 'public'
            else:
                visibility = 'private'

            response.setHeader('Cache-Control',
                               '{}, max-age=31536000'.format(visibility))

        query = self.get_startup_query()
        nodes = self.get_nodes_by_query(
            query, filter_exclude_from_nav=filter_exclude_from_nav)
        nodes = self.prepend_unauthorized_parents(nodes)
        map(partial(self.set_children_loaded_flag, query), nodes)

        return json.dumps(nodes)

    def get_startup_query(self):
        query = self.get_default_query()
        query['path'] = {'query': (list(self.parent_paths_to_nav_root()) +
                                   list(self.get_toplevel_paths())),
                         'depth': 4}
        return query

    def get_startup_cachekey(self):
        cachekey = hashlib.md5()
        query = self.get_startup_query()
        query['sort_on'] = 'modified'
        query['sort_order'] = 'reverse'
        query['sort_limit'] = 50
        brains = self.get_brains(query)[:query['sort_limit']]
        map(cachekey.update, map(str, map(itemgetter('modified'), brains)))
        cachekey.update(api.user.get_current().getId() or '')
        cachekey.update(get_distribution('ftw.mobile').version)
        return cachekey.hexdigest()

    def children(self):
        """Return all nodes
        """
        response = self.request.response
        response.setHeader('Content-Type', 'application/json')
        response.setHeader('X-Theme-Disabled', 'True')

        query = self.get_default_query()
        query['path'] = {'query': ['/'.join(self.context.getPhysicalPath())],
                         'depth': int(self.request.get('depth', 2))}
        return json.dumps(self.get_nodes_by_query(query))

    def parent_paths_to_nav_root(self):
        """Generator of the paths of all parents up to the navigation_root.
        """
        for obj in self.context.aq_chain:
            if hasattr(aq_base(obj), 'getPhysicalPath'):
                yield '/'.join(obj.getPhysicalPath())
            if INavigationRoot.providedBy(obj):
                return

    def prepend_unauthorized_parents(self, nodes):
        """For the navigation to work properly, we need to make sure that
        the parents are included on startup.
        Otherwise no tree can be built and the navigation would not work.
        There might be the problem that the current user is not authorized
        to access a parent (View permission).
        For beeing consistent with other Plone navigations such as the
        navigation portlet or breadcrumbs we want to render the parents
        anyway and let the user hit an "Anauthorized"-error when the user
        clicks.
        This has the advantage that we can build the navigation tree properly.

        This method prepends the missing parents by doing an unrestricted
        catalog query.
        """

        nodes_paths = map(itemgetter('absolute_path'), nodes)
        paths_to_check = list(self.parent_paths_to_nav_root())

        # Append current context path also, because the current obj may
        # be excluded from nav.
        paths_to_check.append('/'.join(self.context.getPhysicalPath()))

        missing_paths = filter(lambda path: path not in nodes_paths,
                               paths_to_check)

        if not missing_paths:
            return nodes

        query = self.get_default_query()
        query['path'] = {'query': missing_paths,
                         'depth': 0}
        parents = self.get_nodes_by_query(query, unrestricted_search=True,
                                          filter_exclude_from_nav=False)
        return parents + nodes

    def get_toplevel_paths(self):
        navroot = api.portal.get_navigation_root(self.context)
        for child in navroot.getFolderContents():
            yield child.getPath()

    def get_nodes_by_query(self, query, unrestricted_search=False,
                           filter_exclude_from_nav=True):
        nodes = map(self.brain_to_node,
                    self.get_brains(
                        query,
                        unrestricted_search=unrestricted_search,
                        filter_exclude_from_nav=filter_exclude_from_nav))
        return nodes

    def get_brains(self, query, unrestricted_search=False,
                   filter_exclude_from_nav=True):
        catalog = api.portal.get_tool('portal_catalog')

        if unrestricted_search:
            brains = catalog.unrestrictedSearchResults(query)
        else:
            brains = catalog.searchResults(query)

        warnsize = 5000
        if len(brains) > warnsize:
            LOG.warning('Query results in more than {} results ({})'
                        .format(warnsize, len(brains)))

        if filter_exclude_from_nav:
            brains = [brain for brain in brains if not brain.exclude_from_nav]

        return brains

    def get_default_query(self):
        portal_types = api.portal.get_tool('portal_types')
        portal_properties = api.portal.get_tool('portal_properties')
        navtree_properties = getattr(portal_properties, 'navtree_properties')

        exclude_types = getattr(navtree_properties, 'metaTypesNotToList', None)
        include_types = list(set(portal_types.keys()) - set(exclude_types))

        sort_on = getattr(navtree_properties,
                          'sortAttribute',
                          'getObjPositionInParent')

        sort_order = getattr(navtree_properties,
                             'sortOrder',
                             'asc')

        query = {'portal_type': include_types,
                 'sort_on': sort_on,
                 'sort_order': sort_order,
                 'is_default_page': False}
        return query

    def brain_to_node(self, brain):
        return {'title': brain.Title,
                'id': brain.id,
                'description': brain.Description,
                'url': get_view_url(brain)[1],
                'absolute_path': brain.getPath(),
                'externallink': is_external_link(brain)}

    def set_children_loaded_flag(self, query, node):
        if not isinstance(query.get('path', None), dict) \
           or 'depth' not in query.get('path', {}):
            # Since we have no path depth limitation we assume that all
            # items were provided in a single response, thus
            # all children are assumed to be loaded.
            node['children_loaded'] = True
            return

        depth = query['path']['depth']
        if depth == -1:
            # Since we have no path depth limitation we assume that all
            # items were provided in a single response, thus
            # all children are assumed to be loaded.
            node['children_loaded'] = True
            return

        path_partials = node['absolute_path'].split('/')
        for _ in range(depth - 1):
            if '/'.join(path_partials) in query['path']['query']:
                node['children_loaded'] = True
                return

            path_partials.pop()



from setuptools import setup, find_packages
import os

version = '1.10.0'
maintainer = 'Mathias Leimgruber'

tests_require = [
    'Products.CMFPlone',
    'ftw.builder',
    'ftw.testbrowser>=1.23.0',
    'ftw.testing',
    'transaction',
    'plone.app.multilingual',
    'unittest2',
]

extras_require = {
    'tests': tests_require,
    'test': tests_require,
}


setup(name='ftw.mobile',
      version=version,
      description='Mobile navigation and a basic infrastructure'
      ' for mobile buttons on a Plone site.',
      long_description=open('README.rst').read() + '\n' +
      open(os.path.join('docs', 'HISTORY.txt')).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          'Framework :: Plone',
          'Framework :: Plone :: 4.3',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Topic :: Software Development :: Libraries :: Python Modules',
      ],

      keywords='ftw plone mobile navigation',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='https://github.com/4teamwork/ftw.mobile',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw', ],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
          'AccessControl',
          'Acquisition',
          'Persistence',
          'Plone',
          'Products.CMFCore',
          'Zope2',
          'ftw.gopip',
          'ftw.theming >= 2.0.0',
          'ftw.upgrade',
          'plone.api',
          'plone.uuid',
          'setuptools',
          'zExceptions',
          'zope.annotation',
          'zope.component',
          'zope.event',
          'zope.i18nmessageid',
          'zope.interface',
          'zope.lifecycleevent',
          'zope.publisher',
      ],

      tests_require=tests_require,
      extras_require=extras_require,

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )
